/* $Id$ -*- C++ -*-
 */

#include "contour.h"

const double DTOR = M_PI / 180;

double distance(const Station& s1, const Station& s2)
{
  double R = 6371; // km
  double dLat = s2._latitude - s1._latitude;
  double dLon = s2._longitude - s1._longitude;

  double a = sin(dLat/2) * sin(dLat/2) +
    cos(s1._latitude) * cos(s2._latitude) * sin(dLon/2) * sin(dLon/2); 
  double c = 2 * atan2(sqrt(a), sqrt(1-a)); 
  return R * c;
}

Angle angle(const Station& s1, const Station& s2)
{
  double dLon = s2._longitude - s1._longitude;

  double y = sin(dLon) * cos(s2._latitude);
  double x = cos(s1._latitude)*sin(s2._latitude)-sin(s1._latitude)*cos(s2._latitude)*cos(dLon);
  return atan2(y, x)/DTOR;
}

void do_it(Station& s1, Station& s2)
{
  double distance = ::distance(s1,s2);
  Angle angle = ::angle(s1, s2);

  std::cout << "distance=" << distance << " bearing from a to b =" << angle << '\n';
  std::cout << "bear-ab\tbear-ba\ta-prot\tb-int\tap-prot\tbp-int\tmargin\n";
  for (double offset = -85; offset <= 85; offset += 5) {
    Angle a1 = angle + offset;
    Radial r1(s1[a1]);

    double offset2 = atan2(           r1._p60 * sin(offset * DTOR),
			   distance - r1._p60 * cos(offset * DTOR)) / DTOR;
    Angle  a2 = angle + 180 - offset2;
    Radial r2(s2[a2]);

    double protected_raw   = r1._p60;
    double interfering_raw = 0;
    switch (std::abs(s1._channel - s2._channel)) {
    case 0:  interfering_raw = r2._i40;  break;
    case 1:  interfering_raw = r2._i54;  break;
    case 2:  interfering_raw = r2._i100; break;
    default: interfering_raw = r2._p60;  break;
    };

    double protected_projected   = protected_raw   * cos(offset  * DTOR);
    double interfering_projected = interfering_raw * cos(offset2 * DTOR);

    std::cout.precision(4);
    std::cout <<         a1
	      << '\t' << a2
	      << '\t' << r1._p60
	      << '\t' << interfering_raw
	      << '\t' << protected_projected
	      << '\t' << interfering_projected
	      << '\t' << distance - (protected_projected + interfering_projected)
	      << "\n";
  }
}


int main(int argc, char* argv[])
{
  std::ifstream f1(argv[1]);
  Station s1(f1);

  std::ifstream f2(argv[2]);
  Station s2(f2);

  std::cout << "margin for interference to " << argv[1] << " from " << argv[2] << '\n';
  do_it(s1, s2);

  std::cout << "\nmargin for interference to " << argv[2] << " from " << argv[1] << '\n';
  do_it(s2, s1);

  return 0;
}
